@@ -8,7 +8,7 @@
   See the file COPYING.LIB.
 */
 
-#include "fuse_config.h"
+#include "config.h"
 #include "fuse_lowlevel.h"
 #include "fuse_misc.h"
 #include "fuse_kernel.h"
@@ -24,24 +24,15 @@
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <assert.h>
-#include <limits.h>
 
 /* Environment var controlling the thread stack size */
 #define ENVNAME_THREAD_STACK "FUSE_THREAD_STACK"
 
-#define FUSE_LOOP_MT_V2_IDENTIFIER	 INT_MAX - 2
-#define FUSE_LOOP_MT_DEF_CLONE_FD	 0
-#define FUSE_LOOP_MT_DEF_MAX_THREADS 10
-#define FUSE_LOOP_MT_DEF_IDLE_THREADS -1 /* thread destruction is disabled
-                                          * by default */
-
-/* an arbitrary large value that cannot be valid */
-#define FUSE_LOOP_MT_MAX_THREADS      (100U * 1000)
-
 struct fuse_worker {
 	struct fuse_worker *prev;
 	struct fuse_worker *next;
 	pthread_t thread_id;
+	size_t bufsize;
 
 	// We need to include fuse_buf so that we can properly free
 	// it when a thread is terminated by pthread_cancel().
@@ -61,7 +52,6 @@ struct fuse_mt {
 	int error;
 	int clone_fd;
 	int max_idle;
-	int max_threads;
 };
 
 static struct fuse_chan *fuse_chan_new(int fd)
@@ -75,7 +65,7 @@ static struct fuse_chan *fuse_chan_new(int fd)
 	memset(ch, 0, sizeof(*ch));
 	ch->fd = fd;
 	ch->ctr = 1;
-	pthread_mutex_init(&ch->lock, NULL);
+	fuse_mutex_init(&ch->lock);
 
 	return ch;
 }
@@ -166,7 +156,7 @@ static void *fuse_do_work(void *data)
 
 		if (!isforget)
 			mt->numavail--;
-		if (mt->numavail == 0 && mt->numworker < mt->max_threads)
+		if (mt->numavail == 0)
 			fuse_loop_start_thread(mt);
 		pthread_mutex_unlock(&mt->lock);
 
@@ -175,14 +165,7 @@ static void *fuse_do_work(void *data)
 		pthread_mutex_lock(&mt->lock);
 		if (!isforget)
 			mt->numavail++;
-
-		/* creating and destroying threads is rather expensive - and there is
-		 * not much gain from destroying existing threads. It is therefore
-		 * discouraged to set max_idle to anything else than -1. If there
-		 * is indeed a good reason to destruct threads it should be done
-		 * delayed, a moving average might be useful for that.
-		 */
-		if (mt->max_idle != -1 && mt->numavail > mt->max_idle && mt->numworker > 1) {
+		if (mt->numavail > mt->max_idle) {
 			if (mt->exit) {
 				pthread_mutex_unlock(&mt->lock);
 				return NULL;
@@ -214,10 +197,7 @@ int fuse_start_thread(pthread_t *thread_id, void *(*func)(void *), void *arg)
 	pthread_attr_t attr;
 	char *stack_size;
 
-	/* Override default stack size
-	 * XXX: This should ideally be a parameter option. It is rather
-	 *      well hidden here.
-	 */
+	/* Override default stack size */
 	pthread_attr_init(&attr);
 	stack_size = getenv(ENVNAME_THREAD_STACK);
 	if (stack_size && pthread_attr_setstacksize(&attr, atoi(stack_size)))
@@ -324,25 +304,12 @@ static void fuse_join_worker(struct fuse_mt *mt, struct fuse_worker *w)
 	free(w);
 }
 
-int fuse_session_loop_mt_312(struct fuse_session *se, struct fuse_loop_config *config);
-FUSE_SYMVER("fuse_session_loop_mt_312", "fuse_session_loop_mt@@FUSE_3.12")
-int fuse_session_loop_mt_312(struct fuse_session *se, struct fuse_loop_config *config)
+FUSE_SYMVER("fuse_session_loop_mt_32", "fuse_session_loop_mt@@FUSE_3.2")
+int fuse_session_loop_mt_32(struct fuse_session *se, struct fuse_loop_config *config)
 {
-int err;
+	int err;
 	struct fuse_mt mt;
 	struct fuse_worker *w;
-	int created_config = 0;
-
-	if (config) {
-		err = fuse_loop_cfg_verify(config);
-		if (err)
-			return err;
-	} else {
-		/* The caller does not care about parameters - use the default */
-		config = fuse_loop_cfg_create();
-		created_config = 1;
-	}
-
 
 	memset(&mt, 0, sizeof(struct fuse_mt));
 	mt.se = se;
@@ -351,11 +318,10 @@ int err;
 	mt.numworker = 0;
 	mt.numavail = 0;
 	mt.max_idle = config->max_idle_threads;
-	mt.max_threads = config->max_threads;
 	mt.main.thread_id = pthread_self();
 	mt.main.prev = mt.main.next = &mt.main;
 	sem_init(&mt.finish, 0, 0);
-	pthread_mutex_init(&mt.lock, NULL);
+	fuse_mutex_init(&mt.lock);
 
 	pthread_mutex_lock(&mt.lock);
 	err = fuse_loop_start_thread(&mt);
@@ -382,105 +348,15 @@ int err;
 	if(se->error != 0)
 		err = se->error;
 	fuse_session_reset(se);
-
-	if (created_config) {
-		fuse_loop_cfg_destroy(config);
-		config = NULL;
-	}
-
 	return err;
 }
 
-int fuse_session_loop_mt_32(struct fuse_session *se, struct fuse_loop_config_v1 *config_v1);
-FUSE_SYMVER("fuse_session_loop_mt_32", "fuse_session_loop_mt@FUSE_3.2")
-int fuse_session_loop_mt_32(struct fuse_session *se, struct fuse_loop_config_v1 *config_v1)
-{
-	int err;
-	struct fuse_loop_config *config = NULL;
-
-	if (config_v1 != NULL) {
-		/* convert the given v1 config */
-		config = fuse_loop_cfg_create();
-		if (config == NULL)
-			return ENOMEM;
-
-		fuse_loop_cfg_convert(config, config_v1);
-	}
-
-	err = fuse_session_loop_mt_312(se, config);
-
-	fuse_loop_cfg_destroy(config);
-
-	return err;
-}
-
-
 int fuse_session_loop_mt_31(struct fuse_session *se, int clone_fd);
 FUSE_SYMVER("fuse_session_loop_mt_31", "fuse_session_loop_mt@FUSE_3.0")
 int fuse_session_loop_mt_31(struct fuse_session *se, int clone_fd)
 {
-	struct fuse_loop_config *config = fuse_loop_cfg_create();
-	if (clone_fd > 0)
-		 fuse_loop_cfg_set_clone_fd(config, clone_fd);
-	return fuse_session_loop_mt_312(se, config);
+	struct fuse_loop_config config;
+	config.clone_fd = clone_fd;
+	config.max_idle_threads = 10;
+	return fuse_session_loop_mt_32(se, &config);
 }
-
-struct fuse_loop_config *fuse_loop_cfg_create(void)
-{
-	struct fuse_loop_config *config = calloc(1, sizeof(*config));
-	if (config == NULL)
-		return NULL;
-
-	config->version_id       = FUSE_LOOP_MT_V2_IDENTIFIER;
-	config->max_idle_threads = FUSE_LOOP_MT_DEF_IDLE_THREADS;
-	config->max_threads      = FUSE_LOOP_MT_DEF_MAX_THREADS;
-	config->clone_fd         = FUSE_LOOP_MT_DEF_CLONE_FD;
-
-	return config;
-}
-
-void fuse_loop_cfg_destroy(struct fuse_loop_config *config)
-{
-	free(config);
-}
-
-int fuse_loop_cfg_verify(struct fuse_loop_config *config)
-{
-	if (config->version_id != FUSE_LOOP_MT_V2_IDENTIFIER)
-		return -EINVAL;
-
-	return 0;
-}
-
-void fuse_loop_cfg_convert(struct fuse_loop_config *config,
-			   struct fuse_loop_config_v1 *v1_conf)
-{
-	fuse_loop_cfg_set_idle_threads(config, v1_conf->max_idle_threads);
-
-	fuse_loop_cfg_set_clone_fd(config, v1_conf->clone_fd);
-}
-
-void fuse_loop_cfg_set_idle_threads(struct fuse_loop_config *config,
-				    unsigned int value)
-{
-	if (value > FUSE_LOOP_MT_MAX_THREADS) {
-		fuse_log(FUSE_LOG_ERR,
-			 "Ignoring invalid max threads value "
-			 "%u > max (%u).\n", value, FUSE_LOOP_MT_MAX_THREADS);
-		return;
-	}
-	config->max_idle_threads = value;
-}
-
-void fuse_loop_cfg_set_max_threads(struct fuse_loop_config *config,
-				   unsigned int value)
-{
-	config->max_threads = value;
-}
-
-void fuse_loop_cfg_set_clone_fd(struct fuse_loop_config *config,
-				unsigned int value)
-{
-	config->clone_fd = value;
-}
-
